function [cmp,snr]=antcode(t0,obs,iobs,nav,inav,ru,opt)
% antcode.m :
% estimate antenna parameters with code observations.
%
%            Copyright(c) 2007-2008 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/03 01:26:51 $
% history: 2008/05/23 0.1 new

C=299792458; f=[1.57542E9;1.2276E9];
for i=1:2, obs(:,i)=C/f(i)*obs(:,i); end % cycle->m
nav=[nav,(datenum(1980,1,6)+nav(:,28)*7-t0)*86400+nav(:,18)]; % toe relative to t0

% ionospheric delay variation with reference antenna
i=find(iobs(:,3)==2);
iono=ionovar(obs(i,:),iobs(i,:));

% code multipath and snr with target antenna/receiver
i=find(iobs(:,3)==1);
obs=obs(i,:); iobs=iobs(i,:);

cmp=[]; snr=[]; mp=[];
for sat=1:32
    i=find(iobs(:,2)==sat);
    j=find(diff(iobs(i,1))>30); % arc break
    for k=[1 j'+1;j' length(i)]
        n=i(k(1):k(2)); if isempty(n), continue; end
        disp(sprintf('TIME=%6.0f-%6.0f : PRN%2d N=%5d',iobs(i(k(1:2)),1),sat,length(n)));
        mp=[mp;multip(obs(n,:),iobs(n,:),nav,inav,iono{sat},opt,ru)];
    end
end
if ~isempty(mp), cmp=mp(:,1:5); snr=mp(:,[1:4,6]); end

% ionospheric delay variations -------------------------------------------------
function iono=ionovar(obs,iobs)
for sat=1:32
    iono{sat}={};
    i=find(iobs(:,2)==sat);
    j=find(diff(iobs(i,1))>30); % arc break
    for k=[1 j'+1;j' length(i)]
        n=i(k(1):k(2));
        ions=ionov(obs(n,:),iobs(n,:));
        iono{sat}={iono{sat}{:},ions{:}};
    end
end

% ionospheric delay variation of each arc --------------------------------------
function ions=ionov(obs,iobs)
f=[1.57542E9;1.2276E9];
ion=obs(:,1:2)*[-1;1]/(1-f(1)^2/f(2)^2); % L1 iono (m)
ions={};
i=find(~isnan(ion));
i=i(abs(diff(ion(i)))>0.03);
i=unique([i;find(iobs(:,4)>=1)-1]); % cycle slip
for j=[1 i'+1;i' length(ion)]
    t=iobs(j(1):j(2),1); 
    ione=ion(j(1):j(2));
    k=find(~isnan(ione)); if length(k)<2, continue; end
    ione=ione-mean(ione(k));
    ione=interp1(t(k),ione(k),t);
    ions={ions{:},[t,ione]};
end

% code multipath and snr -------------------------------------------------------
function mp=multip(obs,iobs,nav,inav,iono,opt,ru)
f=[1.57542E9;1.2276E9]; if opt.f==1, fact=1; else fact=f(1)^2/f(2)^2; end
mp=[]; [pos,E]=togeod(ru);
gf=obs(:,opt.f+2)-obs(:,opt.f); % code-phase
i=find(~isnan(gf));
i=i(abs(diff(gf(i)))>3);
i=unique([i;find(iobs(:,4)>=1)-1]); % cycle slip
for j=[1 i'+1;i' length(gf)]
    k=j(1):j(2);
    t=iobs(k,1); mp0=gf(k); obsk=obs(k,:); iobsk=iobs(k,:);
    for k=1:length(iono)
        [tt,n,m]=intersect(round(t),round(iono{k}(:,1)));
        if length(n)<2, continue; end
        mp1=mp0(n)-2*fact*iono{k}(m,2); if all(isnan(mp1)), continue; end
        mp1=mp1-mean(mp1(~isnan(mp1)));
        azel=sazel(obsk(n,:),iobsk(n,:),nav,inav,ru,E);
        mp=[mp;iobsk(n,1:2),azel,mp1,obsk(n,opt.f+4)];
    end
end

% satellite azimuth/elevation angle --------------------------------------------
function azel=sazel(obs,iobs,nav,inav,ru,E)
azel=repmat(nan,size(obs,1),2);
for i=1:size(obs,1)
    rs=ephpos(obs(i,:),iobs(i,:),nav,inav);
    [r,e]=georng(rs,ru);
    a=E*e;
    azel(i,:)=[atan2(a(1),a(2)),asin(a(3))];
end
