function [x,P,s]=antphas(t0,obs,iobs,nav,inav,ionp,opt)
% antphas.m :
% estimate antenna parameters with carrier-phase observations.
%
%            Copyright(c) 2007-2008 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/03 01:26:51 $
% history: 2008/05/23 0.1 new

C=299792458; f=[1.57542E9;1.2276E9];
for i=1:2, obs(:,i)=C/f(i)*obs(:,i); end % cycle->m
nav=[nav,(datenum(1980,1,6)+nav(:,28)*7-t0)*86400+nav(:,18)]; % toe relative to t0

n=length(sphfun([0 0],opt.nmax));
x=repmat(nan,3+32+n,1); P=zeros(length(x)); s=[]; nout=0; out=zeros(32,1);
[time,i]=unique(round(iobs(:,1))); i=[1;i+1];
for j=1:length(time)
    k=i(j):i(j+1)-1;
    [x,P,out]=udstate(x,P,obs(k,:),iobs(k,:),nav,inav,ionp,out,opt);
    [v,H,R,w]=residuals(time(j),obs(k,:),iobs(k,:),nav,inav,x,opt);
    if isempty(v), continue; end
    K=P*H'/(H*P*H'+R); xp=x+K*v; Pp=P-K*H*P;
    [v,H,R,w]=residuals(time(j),obs(k,:),iobs(k,:),nav,inav,xp,opt);
    rms=sqrt(v'*v/length(v));
    if rms>10*opt.sig
        disp(sprintf('TIME=%6.0f : NS=%2d RMS=%6.3fm *',time(j),length(v),rms));
        nout=nout+1; if nout>10, x(3+(1:32))=nan; nout=0; end
        continue;
    end
    x=xp; P=Pp; s=[s;w]; nout=0;
    if mod(time(j(1))+0.01,300)>0.02, continue; end
    disp(sprintf('TIME=%6.0f : NS=%2d RMS=%6.3fm',time(j),length(v),rms));
end

% double difference residuals -------------------------------------------------
function [v,H,R,w]=residuals(t,obs,iobs,nav,inav,x,opt)
[rs,dts]=ephpos(obs,iobs,nav,inav); [pos,E]=togeod(x(1:3));
n=0;
for sat=unique(iobs(:,2))'
    i=find(iobs(:,2)==sat&iobs(:,3)==1); if isempty(i), continue; end
    j=find(iobs(:,2)==sat&iobs(:,3)==2); if isempty(j), continue; end
    if any(isnan(obs([i,j],opt.f)))|any(obs([i,j],opt.f+4)<opt.minsn), continue; end
    [ri,ei]=georng(rs(:,i),x(1:3));
    [rj,ej]=georng(rs(:,j),opt.rr);
    a=E*ei; azel=[atan2(a(1),a(2)),asin(a(3))];
    if isnan(ri)|isnan(rj)|azel(2)<opt.minel, continue; end
    c=sphfun(azel+[-opt.aza,0],opt.nmax); n=n+1;
    v(n,1)=(obs(i,opt.f)-obs(j,opt.f))-(ri-rj+x(3+sat)+c*x(3+32+1:end));
    H(n,:)=zeros(1,length(x)); H(n,[1:3,3+sat,3+32+1:end])=[-ei' 1 c];
    R(n,1)=2*opt.sig^2; w(n,:)=[t,sat,azel];
end
if n<2, v=[]; H=[]; R=[]; w=[]; return; end
w=[w,v-mean(v)]; D=[ones(n-1,1),-eye(n-1)]; v=D*v; H=D*H; R=D*diag(R)*D';

% update states ----------------------------------------------------------------
function [x,P,out]=udstate(x,P,obs,iobs,nav,inav,ionp,out,opt)
if ~isempty(opt.ru)
    [x,P]=setstate(x,P,1:3,opt.ru,1E-4); % constraint on position
elseif isnan(x(1))
    i=find(iobs(:,3)==1);
    ru=pntpos(obs(i,:),iobs(i,:),nav,inav,ionp,opt.minel);
    [x,P]=setstate(x,P,1:3,ru,10);
end
out=out+1; b=repmat(nan,32,1); 
for sat=unique(iobs(:,2))'
    i=find(iobs(:,2)==sat&iobs(:,3)==1); if isempty(i), continue; end
    j=find(iobs(:,2)==sat&iobs(:,3)==2); if isempty(j), continue; end
    out(sat)=0;
    b(sat)=(obs(i,opt.f)-obs(j,opt.f))-(obs(i,opt.f+2)-obs(j,opt.f+2));
    if iobs(i,opt.f+3)|iobs(j,opt.f+3), x(3+sat)=nan; end % cycle slip
end
for i=find(out>=opt.outc)'
    x(3+i)=nan; % outage count overflow
end
db=b-x(3+(1:32)); i=find(~isnan(db));
if length(i)>3
    if abs(mean(db(i)))>1000&std(db(i))<10
        disp(sprintf('TIME=%6.0f : RCV CLK JUMP DT=%.1fm',iobs(1,1),mean(db(i))));
        x(3+(1:32))=x(3+(1:32))+mean(db(i)); % phase-code coherency
    else
        for j=find(abs(db(i))>20)'
            disp(sprintf('TIME=%6.0f : SLIP PRN%02d',iobs(1,1),i(j)));
            x(3+i(j))=nan;
        end
    end
end
for i=find(~isnan(b)&isnan(x(3+(1:32))))' % reset bias
    %disp(sprintf('TIME=%6.0f : RESET BIAS PRN%02d',iobs(1,1),i));
    [x,P]=setstate(x,P,3+i,b(i),10);
end
if isnan(x(3+32+1))
    n=length(x)-(3+32); i=3+32+(1:n);
    if ~isempty(opt.pcv)&n==length(pcv), x(i)=pcv; P(i,i)=pcvv;
    else [x,P]=setstate(x,P,i,zeros(1,n),1); end
end

% set states ------------------------------------------------------------------
function [x,P]=setstate(x,P,i,xi,sig)
x(i)=xi; P(i,:)=0; P(:,i)=0; P(i,i)=eye(length(i))*sig^2;
