function [rs,dts]=ephpos(obs,iobs,nav,inav)
% ephpos.m :
% compute satellite position/clock biases with ephemeris.
%
%            Copyright(c) 2007-2008 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/03 01:26:51 $
% history: 2008/05/31 1.0 new

C=299792458;
rs=repmat(nan,3,size(obs,1)); dts=repmat(nan,1,size(obs,1));
for i=1:size(obs,1)
    eph=seleph(iobs(1,1),nav,inav,iobs(i,2));
    if isempty(eph)|isnan(obs(i,3)), continue; end
    [rs(:,i),dts(i)]=eph2pos(iobs(i,1)-obs(i,3)/C,eph);
end

% select ephemeris -------------------------------------------------------------
function eph=seleph(t,nav,inav,sat)
eph=[]; i=find(inav==sat); if isempty(i), return, end
[tt,j]=min(abs(t-nav(i,end)));
if tt<=7200&nav(i(j),31)==0, eph=nav(i(j),:); end

% compute satellite position and clock-bias ------------------------------------
function [rs,dts]=eph2pos(t,eph)
C=299792458; OMGE=7.2921151467E-5; MU=3.986005E14;
tk=t-eph(end); tk=t-eph(7:9)*[1;tk;tk^2]-eph(end);
A=eph(17)^2; e=eph(15); n=sqrt(MU/A^3)+eph(12); M=eph(13)+n*tk;
E=M; Ek=0; while abs(E-Ek)>1E-12, Ek=E; E=M+e*sin(Ek); end
p=atan2(sqrt(1-e^2)*sin(E),cos(E)-e)+eph(24);
r=[p;A*(1-e*cos(E));eph(22)+eph(26)*tk]+eph([16,14;11,23;21,19])*[sin(2*p);cos(2*p)];
OMG=eph(20)+(eph(25)-OMGE)*tk-OMGE*eph(18);
rs=r(2)*Rz(-OMG)*Rx(-r(3))*[cos(r(1));sin(r(1));0];
dts=eph(7:9)*[1;tk;tk^2]-2*sqrt(MU*A)*e*sin(E)/C^2-eph(32);

% coordinate rotation matrix ---------------------------------------------------
%function R=Rx(t), c=cos(t); s=sin(t); R=[1,0,0;0,c,s;0,-s,c];
%function R=Rz(t), c=cos(t); s=sin(t); R=[c,s,0;-s,c,0;0,0,1];
function R=Rx(t), R=[1,0,0;0,cos(t),sin(t);0,-sin(t),cos(t)];
function R=Rz(t), R=[cos(t),sin(t),0;-sin(t),cos(t),0;0,0,1];
